/* $Id: euro2.c,v 1.5 1998/03/16 18:22:54 svmcguir Exp $ */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "path.h"
#include "option.h"
#include "mesh.h"
#include "util.h"

/* Calculates the values of the european option corresponding to the
 * american option we are evaluating. Values returned in  results[time]
 * are the european values at each timeslice.
 */
void euro2(meshData *md, double tolerance, double *result){
  int time, underlier, count;
  double **path, **antiPath, payoff, antiPayoff, *sum, *sumSq, *relError, 
    variance, value, maxError, *growthSum, *growthSqSum, growth;
  
  allocate2dDouble(&path, md->option.pData.times, md->option.pData.underliers);
  allocate2dDouble(&antiPath, md->option.pData.times, 
                   md->option.pData.underliers);
  allocate1dDouble(&sum, md->option.pData.times);
  allocate1dDouble(&sumSq, md->option.pData.times);
  allocate1dDouble(&relError, md->option.pData.times);
  allocate1dDouble(&growthSum, md->option.pData.times);
  allocate1dDouble(&growthSqSum, md->option.pData.times);

  for(time=0; time < md->option.pData.times; time++){
    sum[time] = 0;
    sumSq[time] = 0;
    growthSum[time] = 0;
    growthSqSum[time] = 0;
  }

  for(count=0; count < 1000; count+=2){
    pathPair(&(md->option.pData),path,antiPath);

    for(time=0; time < md->option.pData.times; time++){
      payoff = dMax(0,exp(-time*md->option.riskFreeRate)*
                    optionPayoff(&(md->option), path[time]));
      antiPayoff = dMax(0,exp(-time*md->option.riskFreeRate)*
                        optionPayoff(&(md->option), antiPath[time]));
      sum[time] += payoff + antiPayoff;
      sumSq[time] += payoff*payoff + antiPayoff*antiPayoff;
      if(time!=0){
        growth = (path[time][0]-path[time-1][0])/path[time-1][0];
        growthSum[time] += growth;
        growthSqSum[time] += growth*growth;
        growth = (antiPath[time][0]-antiPath[time-1][0])/antiPath[time-1][0];
        growthSum[time] += growth;
        growthSqSum[time] += growth*growth;
      }
    }
  }

  do{
    pathPair(&(md->option.pData),path,antiPath);

    for(time=0; time < md->option.pData.times; time++){
      payoff = dMax(0,exp(-time*md->option.riskFreeRate)*
                    optionPayoff(&(md->option), path[time]));
      antiPayoff = dMax(0,exp(-time*md->option.riskFreeRate)*
                        optionPayoff(&(md->option), antiPath[time]));
      sum[time] += payoff + antiPayoff;
      sumSq[time] += payoff*payoff + antiPayoff*antiPayoff;
      if(time!=0){
        growth = (path[time][0]-path[time-1][0])/path[time-1][0];
        growthSum[time] += growth;
        growthSqSum[time] += growth*growth;
        growth = (antiPath[time][0]-antiPath[time-1][0])/antiPath[time-1][0];
        growthSum[time] += growth;
        growthSqSum[time] += growth*growth;

      }
    } 
    
    count += 2;

    maxError = 0;
    for(time=0; time < md->option.pData.times; time++){
      value = sum[time]/count;
      variance  = sumSq[time]/count - sum[time]/count*sum[time]/count;
      relError[time] = sqrt(variance/count)/fabs(value);
      if(relError[time] > maxError) maxError = relError[time];
    }
    if(count % 100000 == 0)
      printf("%lg of %lg allowed relative error\n",maxError, tolerance);
  }while(maxError > tolerance);


  printf("count = %i\n",count);
  for(time=0; time < md->option.pData.times; time++){
    result[time] = sum[time]/count;
    if(time != 0) 
      printf("Growth: %lg\t%lg\n",growthSum[time]/count,
             growthSqSum[time]/count - 
             (growthSum[time]/count)*(growthSum[time]/count));
  }
}

